#!/usr/bin/env python2
# -*- coding: utf-8 -*-
'''module for working with credit cards via Heartland Payment Systems'''

__version__   = 0.1
__author__    = "Hans-Christoph Steiner"
__all__ = ['e3reader', 'posgateway']
